--=========================================
-- TYPES OF OBJECTS IN THE MEMORY CACHES
--=========================================
SELECT type
	, sum(single_Pages_kb+multi_pages_kb+single_pages_in_use_kb+multi_pages_in_use_kb)/1024 AS total_cached_plans_mb 
	, SUM(entries_count) AS total_entries_count
	, CASE type
		WHEN 'CACHESTORE_SQLCP' THEN 'Dynamic/Raw SQL Statement Plans'
		WHEN 'CACHESTORE_OBJCP' THEN 'Plans for Stored Procedures, Functions, Triggers'
		WHEN 'CACHESTORE_PHDR' THEN 'Parsed SQL Text for Views, Defaults, Constraints'
		WHEN 'CACHESTORE_XPROC' THEN 'Plans for Extended Stored Procedures'
		ELSE 'Other'
	END AS type_description
FROM sys.dm_os_memory_cache_counters
GROUP BY type
HAVING sum(single_Pages_kb+multi_pages_kb+single_pages_in_use_kb+multi_pages_in_use_kb)/1024 > 0
ORDER BY 2 DESC;

GO



--=======================================
--  HOW MUCH SPACE IN BUFFER POOL PER DB?
--=======================================
SELECT DB_NAME(database_id) AS [Database Name],
	COUNT(*) * 8/1024.0 AS [Cached Size (MB)]
FROM sys.dm_os_buffer_descriptors
WHERE database_id > 4 -- system databases
	AND database_id <> 32767 -- ResourceDB
GROUP BY DB_NAME(database_id)
ORDER BY [Cached Size (MB)] DESC;



--=======================================
-- OPTIMIZE FOR AD HOC WORKLOADS ON?
--=======================================
SELECT name, value, value_in_use, description FROM sys.configurations WHERE name = 'optimize for ad hoc workloads';



--========================================
-- SINGLE USE PLANS IN PLAN CACHE
--========================================
/* Kimberly Tripp-Randal */
/* Read more: http://www.sqlskills.com/blogs/kimberly/post/procedure-cache-and-optimizing-for-adhoc-workloads.aspx#ixzz23f2Q12yZ */

SELECT objtype AS [CacheType]
	, COUNT_BIG(*) AS [Total Plans]
	, SUM(CAST(size_in_bytes AS DECIMAL(18, 2))) / 1024 / 1024 AS [Total MBs]
	, SUM(CAST ((CASE 
					WHEN usecounts = 1 THEN size_in_bytes 
					ELSE 0 
				END) AS DECIMAL(18, 2))) / 1024 / 1024 AS [Total MBs - USE Count 1]
	, SUM (CASE 
				WHEN usecounts = 1 THEN 1 
				ELSE 0 
			END) AS [Total Plans - USE Count 1]
FROM sys.dm_exec_cached_plans
GROUP BY objtype
ORDER BY [Total MBs - USE Count 1] 
GO



--============================================
-- ID & REMOVE SINGLE USE PLANS IN PLAN CACHE 
--============================================
/*
COURTESY OF KIM TRIPP:
http://www.sqlskills.com/blogs/kimberly/post/plan-cache-adhoc-workloads-and-clearing-the-single-use-plan-cache-bloat.aspx
Read more: http://www.sqlskills.com/blogs/kimberly/post/plan-cache-adhoc-workloads-and-clearing-the-single-use-plan-cache-bloat.aspx#ixzz27i4H65Pp
*/
DECLARE @MB DECIMAL(19, 3)
DECLARE @Count BIGINT
DECLARE @StrMB NVARCHAR(20)
 
SELECT  @MB = SUM(CAST((CASE WHEN usecounts = 1 THEN size_in_bytes
                             ELSE 0
                        END) AS DECIMAL(12, 2))) / 1024 / 1024
      , @Count = SUM(CASE WHEN usecounts = 1 THEN 1
                          ELSE 0
                     END)
      , @StrMB = CONVERT(NVARCHAR(20), @MB)
FROM    sys.dm_exec_cached_plans
 
IF @MB > 512 
    DBCC FREESYSTEMCACHE('SQL Plans') 
ELSE 
    RAISERROR ('Only %s MB is allocated to single-use plan cache - no need to clear cache now.', 10, 1, @StrMB)
go 

