USE [iDBA]
GO

CREATE PROCEDURE [dbo].[wait_stats_collection] 
AS

DECLARE @now datetime 
SELECT @now = getdate()

IF NOT EXISTS (SELECT name FROM sys.tables WHERE name = 'wait_hash')
	BEGIN
		CREATE TABLE [dbo].[wait_hash]
		(
			[date_stamp] [datetime] NOT NULL,
			[wait_type] [varchar](1000) NULL,
			[wait_time_s] [bigint] NULL,
			[signal_wait_time_s] [bigint] NULL,
			[resource_wait_time_s] [bigint] NULL,
			[waiting_task_count] [bigint] NULL,
			[avg_resource_wait_time_ms] [bigint] NULL,
			[pct] [decimal](5, 2) NULL,
			[running_pct] [decimal](5, 2) NULL
		)
	END

IF NOT EXISTS (SELECT name FROM sys.tables WHERE name = 'wait_history')
	BEGIN
		CREATE TABLE [dbo].[wait_history]
		(
			[wait_id] [int] NOT NULL,
			[date_stamp] [datetime] NOT NULL,
			[wait_type] [varchar](1000) NULL,
			[wait_time_s] [bigint] NULL,
			[signal_wait_time_s] [bigint] NULL,
			[resource_wait_time_s] [bigint] NULL,
			[waiting_task_count] [bigint] NULL,
			[avg_resource_wait_time_ms] [bigint] NULL,
			[pct] [decimal](5, 2) NULL,
			[running_pct] [decimal](5, 2) NULL
		)

		CREATE CLUSTERED INDEX [CL_wait_history] ON [dbo].[wait_history]
		([date_stamp] ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON, FILLFACTOR = 100)
	END

ELSE
	BEGIN
		--------------------------------------------------------------------------
		-- Clean up history
		--------------------------------------------------------------------------
		DELETE FROM iDBA.dbo.wait_history WHERE date_stamp < DATEADD(d,-45, @now)
	END

IF EXISTS (SELECT name FROM tempdb..sysobjects WHERE name LIKE '%#waits%')
	BEGIN
		DROP TABLE #waits;
	END	


CREATE TABLE #waits (date_stamp datetime, wait_type varchar(1000), 
	wait_time_s bigint, signal_wait_time_s bigint, 
	resource_wait_time_s bigint, waiting_task_count bigint, 
	avg_resource_wait_time_ms bigint, pct decimal(5,2), running_pct decimal(5,2)
	);

DECLARE @service_start datetime
SELECT @service_start = create_date FROM master.sys.databases WHERE name = 'tempdb';

/* Isolate top waits for server instance since last restart or statistics clear */
--DBCC SQLPERF('sys.dm_os_wait_stats', CLEAR);
WITH Waits AS
	(
	SELECT 
		wait_type, 
		wait_time_ms / 1000. AS wait_time_s,
		signal_wait_time_ms / 1000 AS signal_wait_time_s, 
		(wait_time_ms - signal_wait_time_ms) / 1000 AS resource_wait_time_s,
		waiting_tasks_count, 
		CASE waiting_tasks_count
			WHEN 0 THEN 0
			ELSE (wait_time_ms - signal_wait_time_ms)/[waiting_tasks_count]
		END AS avg_resource_wait_time_ms,
		 		
		100. * wait_time_ms / SUM(wait_time_ms) OVER() AS pct,
		ROW_NUMBER() OVER(ORDER BY (wait_time_ms - signal_wait_time_ms) DESC) AS rn
	FROM sys.dm_os_wait_stats
	WHERE wait_type 
		NOT IN (	-- filter out additional irrelevant waits
				'BROKER_TASK_STOP', 'BROKER_RECEIVE_WAITFOR', 
				'BROKER_TO_FLUSH', 'BROKER_TRANSMITTER', 'CHECKPOINT_QUEUE', 
				'CHKPT', 'DISPATCHER_QUEUE_SEMAPHORE', 'CLR_AUTO_EVENT', 
				'CLR_MANUAL_EVENT','FT_IFTS_SCHEDULER_IDLE_WAIT', 'KSOURCE_WAKEUP', 
				'LAZYWRITER_SLEEP', 'LOGMGR_QUEUE', 'MISCELLANEOUS', 'ONDEMAND_TASK_QUEUE',
				'REQUEST_FOR_DEADLOCK_SEARCH', 'SLEEP_TASK', 'TRACEWRITE',
				'SQLTRACE_BUFFER_FLUSH', 'XE_DISPATCHER_WAIT', 'XE_TIMER_EVENT'
				,'BROKER_EVENTHANDLER', 'DIRTY_PAGE_POLL', 'HADR_FILESTREAM_IOMGR_IOCOMPLETION', 
				'SQLTRACE_INCREMENTAL_FLUSH_SLEEP', 'SP_SERVER_DIAGNOSTICS_SLEEP'
				)
		)

--Insert collected values into temporary table:
INSERT INTO #waits
	(
		date_stamp, wait_type, wait_time_s, 
		signal_wait_time_s, resource_wait_time_s, 
		waiting_task_count, avg_resource_wait_time_ms, pct, running_pct
	)
SELECT 
	@now, 
	W1.wait_type,
	CAST(W1.wait_time_s AS DECIMAL(10, 0)) AS wait_time_s,
	CAST(W1.signal_wait_time_s AS DECIMAL(10, 0)) AS signal_wait_time_s,
	CAST(W1.resource_wait_time_s AS DECIMAL(10, 0)) AS resource_wait_time_s,
	W1.waiting_tasks_count,
	W1.avg_resource_wait_time_ms,
	CAST(W1.pct AS DECIMAL(5, 2)) AS pct,
	CAST(SUM(W2.pct) AS DECIMAL(5, 2)) AS running_pct
FROM Waits AS W1
	INNER JOIN Waits AS W2 ON W2.rn <= W1.rn
GROUP BY W1.rn, 
	W1.wait_type, 
	W1.wait_time_s,
	W1.signal_wait_time_s,
	W1.resource_wait_time_s,
	W1.avg_resource_wait_time_ms, 
	W1.waiting_tasks_count,
	W1.pct
HAVING SUM(W2.pct) - W1.pct < 95; -- percentage threshold 


DECLARE @collection_id int
SELECT @collection_id = COALESCE(MAX(wait_id),0) + 1 FROM dbo.wait_history

-----------------------------------------------------------------------
--If this is the first time values have been collected:
-----------------------------------------------------------------------
IF (SELECT COUNT(date_stamp) FROM dbo.wait_hash) = 0
	BEGIN
		--Insert the values into the wait_hash table
		INSERT INTO dbo.wait_hash
			(
				date_stamp, wait_type, wait_time_s, 
				signal_wait_time_s, resource_wait_time_s, 
				waiting_task_count, avg_resource_wait_time_ms, pct, running_pct
			)
		SELECT date_stamp, wait_type, wait_time_s, 
				signal_wait_time_s, resource_wait_time_s, 
				waiting_task_count, avg_resource_wait_time_ms, pct, running_pct
		FROM #waits
			
		
		--Then also insert the values into the wait_history table
		INSERT INTO dbo.wait_history
			(
				wait_id, date_stamp, wait_type, wait_time_s, 
				signal_wait_time_s, resource_wait_time_s, 
				waiting_task_count, avg_resource_wait_time_ms, pct, running_pct
			)
		SELECT @collection_id, date_stamp, wait_type, wait_time_s, 
				signal_wait_time_s, resource_wait_time_s, 
				waiting_task_count, avg_resource_wait_time_ms, pct, running_pct
		FROM #waits
	END
	
ELSE

	BEGIN
		/*---------------------------------------------------------------------
			History already exists.  
			Has there been a service restart since the last collection?  
			Populate History Accordingly
		*/-----------------------------------------------------------------------
		IF (SELECT TOP 1 date_stamp FROM #waits) < @service_start
			BEGIN
				--A restart has occurred at some point since the last collection
				INSERT INTO dbo.wait_history
					(
						wait_id, date_stamp, wait_type, wait_time_s, 
						signal_wait_time_s, resource_wait_time_s, 
						waiting_task_count, avg_resource_wait_time_ms, pct, running_pct
					)
				SELECT @collection_id, WS.date_stamp, WS.wait_type, 
					(WS.wait_time_s + WH.wait_time_s) AS wait_time_s, 
					(WS.signal_wait_time_s + WH.signal_wait_time_s) AS signal_wait_time_s, 
					(WS.resource_wait_time_s + WH.resource_wait_time_s) AS resource_wait_time_s, 
					(WS.waiting_task_count + WH.waiting_task_count) AS waiting_task_count, 
					CASE (WS.waiting_task_count + WH.waiting_task_count)
						WHEN 0 THEN 0
						ELSE ((WS.resource_wait_time_s + WH.resource_wait_time_s)/(WS.waiting_task_count + WH.waiting_task_count))
				END AS avg_resource_wait_time_ms, 
					pct, running_pct
				FROM #waits WS
					LEFT JOIN dbo.wait_history WH ON WS.wait_type = WH.wait_type
				WHERE WH.wait_id = (@collection_id - 1)
			END
		ELSE
			BEGIN
				--No restart has occurred
				INSERT INTO dbo.wait_history
					(
						wait_id, date_stamp, wait_type, wait_time_s, 
						signal_wait_time_s, resource_wait_time_s, 
						waiting_task_count, avg_resource_wait_time_ms, pct, running_pct
					)
				SELECT @collection_id, date_stamp, wait_type, wait_time_s, 
						signal_wait_time_s, resource_wait_time_s, 
						waiting_task_count, avg_resource_wait_time_ms, pct, running_pct
				FROM #waits 
			END
	END

-----------------------------------------------------------------------
--Clean and reload Hash table
-----------------------------------------------------------------------
TRUNCATE TABLE dbo.wait_hash;

INSERT INTO dbo.wait_hash
	(
		date_stamp, wait_type, wait_time_s, 
		signal_wait_time_s, resource_wait_time_s, 
		waiting_task_count, avg_resource_wait_time_ms, pct, running_pct
	)
SELECT date_stamp, wait_type, wait_time_s, 
		signal_wait_time_s, resource_wait_time_s, 
		waiting_task_count, avg_resource_wait_time_ms, pct, running_pct
FROM #waits;


/*
Troubleshooting Only:
SELECT * FROM #waits
SELECT * FROM dbo.wait_hash;
SELECT * FROM dbo.wait_history;
*/

DROP TABLE #waits;






--Look at Signal Wait (> 10-15% means CPU issues)
--SELECT 
--	SUM([signal_wait_time_ms]) AS signal_wait_time_ms,
--	'pct_signal_waits' = cast(100.0 * SUM(signal_wait_time_ms) / SUM (wait_time_ms) as numeric(20,2)),
--	SUM([wait_time_ms]) - SUM([signal_wait_time_ms]) AS resource_wait_time_ms,
--	'pct_resource_waits'= cast(100.0 * SUM(wait_time_ms - signal_wait_time_ms) / SUM (wait_time_ms) as numeric(20,2))
--FROM sys.[dm_os_wait_stats] dows;

/*
TRUNCATE TABLE dbo.wait_hash;
TRUNCATE TABLE dbo.wait_history;
DROP TABLE dbo.wait_hash;
DROP TABLE dbo.wait_history;
*/

